"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const types_1 = require("@babel/types");
const never_1 = tslib_1.__importDefault(require("never"));
const transformNode = (node) => {
    if (types_1.isArrowFunctionExpression(node) ||
        types_1.isFunctionDeclaration(node) ||
        types_1.isTSDeclareFunction(node)) {
        node.params.forEach((param) => {
            const { typeAnnotation } = param;
            if (types_1.isTSTypeAnnotation(typeAnnotation) &&
                !(types_1.isIdentifier(param) && param.optional === true)) {
                if (types_1.isTSUnionType(typeAnnotation.typeAnnotation)) {
                    if (!typeAnnotation.typeAnnotation.types.some(type => types_1.isTSUndefinedKeyword(type))) {
                        typeAnnotation.typeAnnotation.types.push(types_1.tSUndefinedKeyword());
                    }
                }
                else {
                    typeAnnotation.typeAnnotation = types_1.tsUnionType([
                        types_1.isTSFunctionType(typeAnnotation.typeAnnotation)
                            ? types_1.isTSParenthesizedType(typeAnnotation.typeAnnotation)
                                ? typeAnnotation.typeAnnotation
                                : types_1.tsParenthesizedType(typeAnnotation.typeAnnotation)
                            : typeAnnotation.typeAnnotation,
                        types_1.tSUndefinedKeyword()
                    ]);
                }
            }
        });
    }
};
const plugin = {
    visitor: {
        ExportDefaultDeclaration: path => {
            var _a;
            if (types_1.isIdentifier(path.node.declaration)) {
                const { name } = path.node.declaration;
                const { path: { node } } = (_a = path.scope.getBinding(name)) !== null && _a !== void 0 ? _a : never_1.default('No binding');
                if (types_1.isVariableDeclarator(node)) {
                    transformNode(node.init);
                }
            }
            else {
                transformNode(path.node.declaration);
            }
        },
        ExportDeclaration: path => {
            if (types_1.isExportNamedDeclaration(path.node)) {
                if (types_1.isVariableDeclaration(path.node.declaration)) {
                    transformNode(path.node.declaration.declarations[0].init);
                }
            }
        },
        ExportNamedDeclaration: path => {
            if (types_1.isTSDeclareFunction(path.node.declaration)) {
                transformNode(path.node.declaration);
            }
        }
    }
};
exports.default = plugin;
